% P=MLD2PWA(S,xmax,umax,options)
%
% (C) 2002 by A. Bemporad, University of Siena <bemporad@dii.unisi.it>
%
% Given the hybrid MLD system S (obtained e.g. from HYSDEL), returns the PWA system P
%
% xc(t+1)=A(i)xc(t)+B(i)uc(t)+f(i)    x=[xc,xl], u=[uc,ul]
% xl(t+1)=Al(i)xc(t)+Bl(i)u(c)+fl(i)  if Hx(i)x(t)+Hu(i)u(t)<=K(i)
%
%
% which is equivalent to S in the box -xmax <= x(t) <= xmax, -umax <= u(t) <= umax.
% The conversion is based on Dua-Pistikopoulos' state-partition strategy.
%
% P.n = number of regions
% P.A = cell array of A(i) matrices
% P.B = cell array of B(i) matrices
% P.f = cell array of f(i) matrices
% P.Hx = cell array of Hx(i) matrices
% P.Hu = cell array of Hu(i) matrices
% P.K = cell array of K(i) matrices
%
% P.AL = cell array of AL(i) matrices  (Hysdel models: P.AL is always zero)
% P.BL = cell array of BL(i) matrices  (Hysdel models: P.BL is always zero)
% P.fL = cell array of fL(i) matrices  
%
% P.logic = cell array of combinations of (xl,ul,delta) variables
% P.nx = state dimension
% P.nxr = continuous state dimension
% P.nxb = logic state dimension
% P.nu = input dimension
% P.nur = continuous input dimension
% P.nub = logic input dimension
%
% Note that integer states and inputs are included in the x and u vectors, respectively.
%
%
% options.verbose=1: plots intermediate results
%                =0: nothing
%                =2: also show results of MILP
% options.solver='nag'     uses MIQP3_NAF.M based on E04MBF.M for LP
% options.solver='matlab'  uses MIQP3_NAF.M based on LP.M for LP
% options.bounds=1         add bounds -xmax<=x<=xmax, -umax<=u<=umax in the polyhedral
%                          cell definition (=0 don't add)
% options.tighteps         Tolerance which decides if two dynamics (A,B,f) are equal (default:1e-6)
%
% ---------------------------------------------
% How the algorithm works:
%
% 1. Finds the Chebischev center (x*,u*) of the current region in the (xc,xL,uc,uL)-space
%    
% 2. Finds the best approximation of (x*,u*) (in infinity-norm) which is integer
%    feasible and satisfies the MLD constraints, via MILP
%
% 3. If the MILP is infeasible, terminate the exploration of the current region
%
% 4. As a result of the MILP, fix the (xL*,uL*,delta*) variables
%
% 5. Find z(xc,uc), and then A,B,f,AL,BL,fL,Hx,Hu,K. In Hx,Hu,K, also include the
%    constraint xL*-0.5<=xL<=xL*+0.5 (and similarly for uL)
%
% 6. If the combination (xL*,uL*,delta*) is a new one, augment the PWA system with (Hx,Hu,K)
%
% 7. Intersect (Hx,Hu,K) with the current region, partition the rest of the region,
%    go recursively to 1
%
% 8. At the end of the recursion, consider all pairs of regions in the (xc,xL,uc,uL)-space
%    where the dynamics (both continuous and logic) is the same, and try to compute the union.
%
% The procedure is described in the following paper:
%
%@inproceedings{Bem02a,
%  author = {A. Bemporad},
%  title = {An Efficient Technique for Translating Mixed Logical Dynamical Systems
%           into Piecewise Affine Systems},
%  booktitle={Proc. 41th IEEE Conf. on Decision and Control},
%  year = {2002},
%}
